@@ -1,4 +1,4 @@
-'''
+r"""
 \mainpage
 
 This is the mainpage of the smact package
@@ -6,4 +6,4 @@ This is the mainpage of the smact package
 Authors Keith T. Butler, Adam J. Jackson, Daniel Davies, Josh Evans, Tim Gauntlet, Aron Walsh.
 
 Semiconducting Materials from Analogy and Chemical Theory (SMACT)
-'''
+"""
