@@ -3,21 +3,24 @@
 import logging
 import re
 from collections import defaultdict
+from functools import reduce
+from math import gcd
 from operator import itemgetter
 from typing import Dict, List, Optional, Tuple, Union
 
-from math import gcd
-from functools import reduce
-
 import numpy as np
 import pymatgen
 from pymatgen.analysis.bond_valence import BVAnalyzer
 from pymatgen.ext.matproj import MPRester
+from pymatgen.transformations.standard_transformations import (
+    OxidationStateDecorationTransformation,
+)
 
 import smact
 
 from . import logger
-from .utilities import get_sign
+from .utilities import convert_next_gen_mprest_data, get_sign
+
 
 class SmactStructure:
     """SMACT implementation inspired by pymatgen Structure class.
@@ -46,12 +49,12 @@ class SmactStructure:
     """
 
     def __init__(
-      self,
-      species: List[Union[Tuple[str, int, int], Tuple[smact.Species, int]]],
-      lattice_mat: np.ndarray,
-      sites: Dict[str, List[List[float]]],
-      lattice_param: Optional[float] = 1.0,
-      sanitise_species: Optional[bool] = True,
+        self,
+        species: List[Union[Tuple[str, int, int], Tuple[smact.Species, int]]],
+        lattice_mat: np.ndarray,
+        sites: Dict[str, List[List[float]]],
+        lattice_param: Optional[float] = 1.0,
+        sanitise_species: Optional[bool] = True,
     ):
         """Initialize structure with constituent species.
 
@@ -66,11 +69,15 @@ class SmactStructure:
                 :meth:`~.from_mp`.
 
         """
-        self.species = self._sanitise_species(species) if sanitise_species else species
+        self.species = (
+            self._sanitise_species(species) if sanitise_species else species
+        )
 
         self.lattice_mat = lattice_mat
 
-        self.sites = {spec: sites[spec] for spec in self.get_spec_strs()}  # Sort sites
+        self.sites = {
+            spec: sites[spec] for spec in self.get_spec_strs()
+        }  # Sort sites
 
         self.lattice_param = lattice_param
 
@@ -97,17 +104,19 @@ class SmactStructure:
         """
         if not isinstance(other, SmactStructure):
             return False
-        return all([
-          self.species == other.species,
-          np.array_equal(self.lattice_mat, other.lattice_mat),
-          self.lattice_param == other.lattice_param,
-          self.sites == other.sites,
-          list(self.sites.keys()) == list(other.sites.keys()),
-        ])
+        return all(
+            [
+                self.species == other.species,
+                np.array_equal(self.lattice_mat, other.lattice_mat),
+                self.lattice_param == other.lattice_param,
+                self.sites == other.sites,
+                list(self.sites.keys()) == list(other.sites.keys()),
+            ]
+        )
 
     @staticmethod
     def _sanitise_species(
-      species: List[Union[Tuple[str, int, int], Tuple[smact.Species, int]]],
+        species: List[Union[Tuple[str, int, int], Tuple[smact.Species, int]]],
     ) -> List[Tuple[str, int, int]]:
         """Sanitise and format a list of species.
 
@@ -129,12 +138,14 @@ class SmactStructure:
         if len(species) == 0:
             raise ValueError("`species` cannot be empty.")
         if not isinstance(species[0], tuple):
-            raise TypeError(f"`species` must be a list of tuples, got list of {type(species[0])}.")
+            raise TypeError(
+                f"`species` must be a list of tuples, got list of {type(species[0])}."
+            )
 
         species_error = (
-          "`species` list of tuples must contain either "
-          "2-tuples of Species objects and stoichiometries, "
-          "or 3-tuples of elements, oxidations and stoichiometries."
+            "`species` list of tuples must contain either "
+            "2-tuples of Species objects and stoichiometries, "
+            "or 3-tuples of elements, oxidations and stoichiometries."
         )
         if len(species[0]) not in {2, 3}:
             raise ValueError(species_error)
@@ -144,9 +155,13 @@ class SmactStructure:
             species.sort(key=itemgetter(0))
             sanit_species = species
 
-        elif isinstance(species[0][0], smact.Species):  # Species class variation of instantiation
+        elif isinstance(
+            species[0][0], smact.Species
+        ):  # Species class variation of instantiation
             species.sort(key=lambda x: (x[0].symbol, -x[0].oxidation))
-            sanit_species = [(x[0].symbol, x[0].oxidation, x[1]) for x in species]
+            sanit_species = [
+                (x[0].symbol, x[0].oxidation, x[1]) for x in species
+            ]
 
         else:
             raise TypeError(species_error)
@@ -155,7 +170,7 @@ class SmactStructure:
 
     @staticmethod
     def __parse_py_sites(
-      structure: pymatgen.core.Structure,
+        structure: pymatgen.core.Structure,
     ) -> Tuple[Dict[str, List[List[float]]], List[Tuple[str, int, int]]]:
         """Parse the sites of a pymatgen Structure.
 
@@ -173,16 +188,21 @@ class SmactStructure:
 
         """
         if not isinstance(structure, pymatgen.core.Structure):
-            raise TypeError("structure must be a pymatgen.core.Structure instance.")
+            raise TypeError(
+                "structure must be a pymatgen.core.Structure instance."
+            )
 
         sites = defaultdict(list)
         for site in structure.sites:
             site_type = site.species_string
             # Add charge magnitude, for cases of unit charge
-            if all([
-              site_type[-2] not in map(str, range(10)),
-              site_type[-1] in ("+", "-"), ]):
-                site_type = site_type[:-1] + '1' + site_type[-1]
+            if all(
+                [
+                    site_type[-2] not in map(str, range(10)),
+                    site_type[-1] in ("+", "-"),
+                ]
+            ):
+                site_type = site_type[:-1] + "1" + site_type[-1]
 
             sites[site_type].append(site.coords.tolist())
 
@@ -210,21 +230,54 @@ class SmactStructure:
         return sites, species
 
     @staticmethod
-    def from_py_struct(structure: pymatgen.core.Structure):
+    def from_py_struct(
+        structure: pymatgen.core.Structure, determine_oxi: str = "BV"
+    ):
         """Create a SmactStructure from a pymatgen Structure object.
 
         Args:
             structure: A pymatgen Structure.
+            determine_oxi (str): The method to determine the assignments oxidation states in the structure.
+                Options are 'BV', 'comp_ICSD','both' for determining the oxidation states by bond valence,
+                ICSD statistics or trial both sequentially, respectively.
 
         Returns:
             :class:`~.SmactStructure`
 
         """
         if not isinstance(structure, pymatgen.core.Structure):
-            raise TypeError("Structure must be a pymatgen.core.Structure instance.")
+            raise TypeError(
+                "Structure must be a pymatgen.core.Structure instance."
+            )
 
-        bva = BVAnalyzer()
-        struct = bva.get_oxi_state_decorated_structure(structure)
+        if determine_oxi == "BV":
+            bva = BVAnalyzer()
+            struct = bva.get_oxi_state_decorated_structure(structure)
+
+        elif determine_oxi == "comp_ICSD":
+            comp = structure.composition
+            oxi_transform = OxidationStateDecorationTransformation(
+                comp.oxi_state_guesses()[0]
+            )
+            struct = oxi_transform.apply_transformation(structure)
+            print("Charge assigned based on ICSD statistics")
+
+        elif determine_oxi == "both":
+            try:
+                bva = BVAnalyzer()
+                struct = bva.get_oxi_state_decorated_structure(structure)
+                print("Oxidation states assigned using bond valence")
+            except ValueError:
+                comp = structure.composition
+                oxi_transform = OxidationStateDecorationTransformation(
+                    comp.oxi_state_guesses()[0]
+                )
+                struct = oxi_transform.apply_transformation(structure)
+                print("Oxidation states assigned based on ICSD statistics")
+        else:
+            raise ValueError(
+                f"Argument for 'determine_oxi', <{determine_oxi}> is not valid. Choose either 'BV','comp_ICSD' or 'both'."
+            )
 
         sites, species = SmactStructure.__parse_py_sites(struct)
 
@@ -233,20 +286,26 @@ class SmactStructure:
         lattice_param = 1.0
 
         return SmactStructure(
-          species,
-          lattice_mat,
-          sites,
-          lattice_param,
-          sanitise_species=True, )
+            species,
+            lattice_mat,
+            sites,
+            lattice_param,
+            sanitise_species=True,
+        )
 
     @staticmethod
     def from_mp(
-      species: List[Union[Tuple[str, int, int], Tuple[smact.Species, int]]],
-      api_key: str, ):
+        species: List[Union[Tuple[str, int, int], Tuple[smact.Species, int]]],
+        api_key: str,
+        determine_oxi: str = "BV",
+    ):
         """Create a SmactStructure using the first Materials Project entry for a composition.
 
         Args:
             species: See :meth:`~.__init__`.
+            determine_oxi (str): The method to determine the assignments oxidation states in the structure.
+                Options are 'BV', 'comp_ICSD','both' for determining the oxidation states by bond valence,
+                ICSD statistics or trial both sequentially, respectively.
             api_key: A www.materialsproject.org API key.
 
         Returns:
@@ -258,22 +317,57 @@ class SmactStructure:
         with MPRester(api_key) as m:
             eles = SmactStructure._get_ele_stoics(sanit_species)
             formula = "".join(f"{ele}{stoic}" for ele, stoic in eles.items())
-            structs = m.query(
-              criteria={"reduced_cell_formula": formula},
-              properties=["structure"], )
+            try:
+                # Legacy API routine
+                structs = m.query(
+                    criteria={"reduced_cell_formula": formula},
+                    properties=["structure"],
+                )
+            except NotImplementedError:
+                # New API routine
+                docs = m.summary.search(formula=formula, fields=["structure"])
+                structs = [convert_next_gen_mprest_data(doc) for doc in docs]
 
             if len(structs) == 0:
                 raise ValueError(
-                  "Could not find composition in Materials Project Database, "
-                  "please supply a structure."
+                    "Could not find composition in Materials Project Database, "
+                    "please supply a structure."
                 )
 
-            struct = structs[0]['structure']  # Default to first found structure
+            # Default to first found structure
+            struct = structs[0]["structure"]
 
-        if 0 not in (spec[1] for spec in sanit_species):  # If everything's charged
-            bva = BVAnalyzer()
-            struct = bva.get_oxi_state_decorated_structure(struct)
+        if 0 not in (
+            spec[1] for spec in sanit_species
+        ):  # If everything's charged
+            if determine_oxi == "BV":
+                bva = BVAnalyzer()
+                struct = bva.get_oxi_state_decorated_structure(struct)
 
+            elif determine_oxi == "comp_ICSD":
+                comp = struct.composition
+                oxi_transform = OxidationStateDecorationTransformation(
+                    comp.oxi_state_guesses()[0]
+                )
+                struct = oxi_transform.apply_transformation(struct)
+                print("Charge assigned based on ICSD statistics")
+
+            elif determine_oxi == "both":
+                try:
+                    bva = BVAnalyzer()
+                    struct = bva.get_oxi_state_decorated_structure(struct)
+                    print("Oxidation states assigned using bond valence")
+                except ValueError:
+                    comp = struct.composition
+                    oxi_transform = OxidationStateDecorationTransformation(
+                        comp.oxi_state_guesses()[0]
+                    )
+                    struct = oxi_transform.apply_transformation(struct)
+                    print("Oxidation states assigned based on ICSD statistics")
+            else:
+                raise ValueError(
+                    f"Argument for 'determine_oxi', <{determine_oxi}> is not valid. Choose either 'BV','comp_ICSD' or 'both'."
+                )
         lattice_mat = struct.lattice.matrix
 
         lattice_param = 1.0  # TODO Use actual lattice parameter
@@ -281,11 +375,12 @@ class SmactStructure:
         sites, _ = SmactStructure.__parse_py_sites(struct)
 
         return SmactStructure(
-          sanit_species,
-          lattice_mat,
-          sites,
-          lattice_param,
-          sanitise_species=False, )
+            sanit_species,
+            lattice_mat,
+            sites,
+            lattice_param,
+            sanitise_species=False,
+        )
 
     @staticmethod
     def from_file(fname: str):
@@ -299,7 +394,7 @@ class SmactStructure:
             :class:`~.SmactStructure`
 
         """
-        with open(fname, 'r') as f:
+        with open(fname) as f:
             return SmactStructure.from_poscar(f.read())
 
     @staticmethod
@@ -337,7 +432,12 @@ class SmactStructure:
 
         lattice_param = float(lines[1])
 
-        lattice = np.array([[float(point) for point in line.split(" ")] for line in lines[2:5]])
+        lattice = np.array(
+            [
+                [float(point) for point in line.split(" ")]
+                for line in lines[2:5]
+            ]
+        )
 
         sites = defaultdict(list)
         for line in lines[8:]:
@@ -355,10 +455,11 @@ class SmactStructure:
         return SmactStructure(species, lattice, sites, lattice_param)
 
     def _format_style(
-      self,
-      template: str,
-      delim: Optional[str] = " ",
-      include_ground: Optional[bool] = False, ) -> str:
+        self,
+        template: str,
+        delim: Optional[str] = " ",
+        include_ground: Optional[bool] = False,
+    ) -> str:
         """Format a given template string with the composition.
 
         Formats a python template string with species information,
@@ -369,16 +470,16 @@ class SmactStructure:
                 curly brackets notation. Supported keywords are
                 `ele` for the elemental symbol, `stoic` for the
                 stoichiometry, `charge` for the absolute value
-                of oxidation state and `sign` for the 
+                of oxidation state and `sign` for the
                 oxidation state's sign.
             delim: The delimeter between species' templates.
             include_ground: Whether to include the charge and sign
                 of neutral species.
-        
+
         Returns:
             String of templates formatted for each species, separated
                 by `delim`.
-        
+
         Examples:
             >>> s = SmactStructure.from_file('tests/files/CaTiO3.txt')
             >>> template = '{stoic}x{ele}{charge}{sign}'
@@ -388,21 +489,23 @@ class SmactStructure:
         """
         if include_ground:
             return delim.join(
-              template.format(
-                ele=specie[0],
-                stoic=specie[2],
-                charge=abs(specie[1]),
-                sign="+" if specie[1] >= 0 else "-",
-              ) for specie in self.species
+                template.format(
+                    ele=specie[0],
+                    stoic=specie[2],
+                    charge=abs(specie[1]),
+                    sign="+" if specie[1] >= 0 else "-",
+                )
+                for specie in self.species
             )
 
         return delim.join(
-          template.format(
-            ele=specie[0],
-            stoic=specie[2],
-            charge=abs(specie[1]) if specie[1] != 0 else "",
-            sign=get_sign(specie[1]),
-          ) for specie in self.species
+            template.format(
+                ele=specie[0],
+                stoic=specie[2],
+                charge=abs(specie[1]) if specie[1] != 0 else "",
+                sign=get_sign(specie[1]),
+            )
+            for specie in self.species
         )
 
     @staticmethod
@@ -436,14 +539,14 @@ class SmactStructure:
 
         Returns:
             A list of strings, formatted as '{element}{charge}{sign}'.
-        
+
         Examples:
             >>> s = SmactStructure.from_file('tests/files/CaTiO3.txt')
             >>> s.get_spec_strs()
             ['Ca2+', 'O2-', 'Ti4+']
-        
+
         """
-        return self._format_style("{ele}{charge}{sign}").split(' ')
+        return self._format_style("{ele}{charge}{sign}").split(" ")
 
     def composition(self) -> str:
         """Generate a key that describes the composition.
@@ -491,13 +594,21 @@ class SmactStructure:
 
         poscar += f"{self.lattice_param}\n"
 
-        poscar += "\n".join(" ".join(map(str, vec)) for vec in self.lattice_mat.tolist()) + "\n"
+        poscar += (
+            "\n".join(
+                " ".join(map(str, vec)) for vec in self.lattice_mat.tolist()
+            )
+            + "\n"
+        )
 
         spec_count = {spec: len(coords) for spec, coords in self.sites.items()}
 
         poscar += self._format_style("{ele}") + "\n"
 
-        poscar += " ".join(str(spec_count[spec]) for spec in self.get_spec_strs()) + "\n"
+        poscar += (
+            " ".join(str(spec_count[spec]) for spec in self.get_spec_strs())
+            + "\n"
+        )
 
         poscar += "Cartesian\n"
         for spec, coords in self.sites.items():
